/*
 * Chess game implementation.
 * By Carlos Varela.  February, 1997
 *
 * Algorithm class.  Contains the chess algorithm.
 */

package chess;

import java.util.*;

public class Algorithm implements java.io.Serializable {

  static int nodes = 0;   // Number of nodes evaluated.

  /**
   * nodesEvaluated returns and resets the number of evaluated nodes.
   */
  public static int nodesEvaluated(){
    int oldNodes = nodes;
    nodes = 0;
    return oldNodes;
  }

  /**
   * chooseBest receives a chess board, the playing color and
   * a (possibly incomplete) tree of moves, and returns the
   * best move for that player.  It throws an exception if
   * the board is in checkmate or draw, or if the move creates 
   * a checkmate or a draw.
   */
  public static Move chooseBest(Board b, boolean color, MoveTree possibleMoves)
    throws CheckMateException, DrawException {
    try {
      int maxMin = possibleMoves.getMaxMin();
      if (maxMin >= Game.INFINITY)
	throw new CheckMateException(Game.WHITE, possibleMoves.getMaxMinTree());
      if (maxMin <= -Game.INFINITY)
	throw new CheckMateException(Game.BLACK, possibleMoves.getMaxMinTree());
       if (possibleMoves.getMaxMinTree().getMove() instanceof Move)
        return (Move) possibleMoves.getMaxMinTree().getMove();
        else return null;
//       return (((MoveTree)possibleMoves.getNextMoves().elementAt(0)).getMove());
    } catch (NullPointerException npe){ 
      // MaxMinTree is null if there are no possible moves.
      if (b.isCheck(!color))
	throw new CheckMateException(!color);
      else throw new DrawException();
     } catch (ArrayIndexOutOfBoundsException aiouobe){ 
       // NextMoves is null if there are no possible moves.
       if (b.isCheck(!color))
 	throw new CheckMateException(!color);
       else throw new DrawException();
    }
  }

  /*
   * evaluateStaticFunction receives a chess board and
   * returns a static evaluation function on that board.
   * Positive values represent good positions for white pieces.
   * Negative values represent good positions for black pieces.
   * It may throw a CutSearchTreeException, if this particular
   * board position should not be explored any further.  In such
   * case, it includes an appropriate value within the exception.
   */
  public static int evaluateStaticFunction(submit.Board b) throws CutSearchTreeException {
    if (b instanceof Board)
      return evaluateStaticFunction((Board) b);
    return 0;
  }

  public static int evaluateStaticFunction(Board b)
    throws CutSearchTreeException {
    // Incomplete:  Initial game:  see database
    //              Middle game:   see defense/attack structure
    //              Final game:    see final strategies
    nodes++;
    if (b.moveRepetition())
      throw new CutSearchTreeException(0);
    else return (evaluatePieceValues(b.getWhitePiecesOnBoard()) +
		 evaluatePieceValues(b.getBlackPiecesOnBoard()));
  }

  /**
   * evaluatePieceValues takes a vector of pieces and evaluates their value.
   */
  public static int evaluatePieceValues(Vector v){
    // FIX Efficiency:  create/estimate these values while "generateMoves" ONCE.
    // For now:  Sum of piece values (9, 5, 3, 3, 1)
    //          -(Sum of unprotected piece values)/3
    //          +(Number of pieces in four center positions)/4
    //          +(Number of pieces reaching four center positions)/8
    //          +(Sum of pieces attacking opposite king)/2
    int value = 0;
    for (Enumeration pieces = v.elements();
	 pieces.hasMoreElements(); ) {
      Piece p = (Piece) pieces.nextElement();
      value += p.getValue();
//       if (!(p.isProtected())) 
// 	value += p.getValue()/3;
      if (p.isCentered())
	value += (p.color()? 25 : -25);
//        if (p.reachesCenter())
//  	value += (p.color()? 0.12 : -0.12);
//       if (p.makesCheck())
//       	value += p.getValue()/2;
    }
    return value;
  }

}
