/*
 * Chess game implementation.
 * By Carlos Varela.  February, 1997
 *
 * Bishop class.  Contains a chess piece representing a bishop.
 */

package chess;

import java.util.*;

class Bishop extends Piece implements java.io.Serializable  {

  Bishop(boolean col){
    value = (col? 300 : -300);
    color = col;
  }

  Vector generateMoves(){
    Vector bishopMoves = new Vector(13);
    char c = getColumn();
    int r = getRow();
    Board b = getBoard();
    Piece p;
    // Bishop movements
    
                                 // Right Upward moves 
    try {
      for (int i = 1; i <= 7; i++){
	int ri = r + i;
	char ci = (char) (c + i);
	if ((p = b.getPieceAt(ci,ri)) != null){  // A piece is in the way
	  if (p.color() != color())            // Can eat if opposite color.
	    bishopMoves.addElement(new Move(this, c, r, ci, ri));
	  break;
	} else
	  bishopMoves.addElement(new Move(this, c, r, ci, ri));
      }
    } catch (ArrayIndexOutOfBoundsException e){};  /* Out of board */
	 
                                 // Right Downward moves 
    try {
      for (int i = 1; i <= 7; i++){
	int ri = r - i;
	char ci = (char) (c + i);
	if ((p = b.getPieceAt(ci,ri)) != null){  // A piece is in the way
	  if (p.color() != color())            // Can eat if opposite color.
	    bishopMoves.addElement(new Move(this, c, r, ci, ri));
	  break;
	} else
	  bishopMoves.addElement(new Move(this, c, r, ci, ri));
      }
    } catch (ArrayIndexOutOfBoundsException e){};  /* Out of board */

                                 // Left Upward moves 
    try {
      for (int i = 1; i <= 7; i++){
	int ri = r + i;
	char ci = (char) (c - i);
	if ((p = b.getPieceAt(ci,ri)) != null){  // A piece is in the way
	  if (p.color() != color())            // Can eat if opposite color.
	    bishopMoves.addElement(new Move(this, c, r, ci, ri));
	  break;
	} else
	  bishopMoves.addElement(new Move(this, c, r, ci, ri));
      }
    } catch (ArrayIndexOutOfBoundsException e){};  /* Out of board */
	 
                                 // Left Downward moves 
    try {
      for (int i = 1; i <= 7; i++){
	int ri = r - i;
	char ci = (char) (c - i);
	if ((p = b.getPieceAt(ci,ri)) != null){  // A piece is in the way
	  if (p.color() != color())            // Can eat if opposite color.
	    bishopMoves.addElement(new Move(this, c, r, ci, ri));
	  break;
	} else
	  bishopMoves.addElement(new Move(this, c, r, ci, ri));
      }
    } catch (ArrayIndexOutOfBoundsException e){};  /* Out of board */

    return bishopMoves;
  }

  boolean isProtected(){
    // Incomplete.
    return true;
  }

  boolean attacksCenter(){
    // Incomplete.
    return false;
  }

  boolean makesCheck(){
    // Incomplete.
    return false;
  }

  public String toString(){
    return ((color == Game.WHITE)? "B" : "b");
  }

}
