/*
 * Chess game implementation.
 * By Carlos Varela.  February, 1997
 *
 * Clock class.  Contains the clocks for the players.
 */

package chess;

import java.util.*;

public class Clock implements java.io.Serializable {

  boolean stopped;
  long startedAt;
  long currentValue;

  public Clock() {
    stopped = true;
    startedAt = 0;
    currentValue = 0;
  }

  /* 
   * start starts the time passing of the clock.
   */
  public void start(){
    Date d = new Date(); // Use System.currentTimeMillis();
    if (stopped){
      stopped = false;
      startedAt = d.getTime() - currentValue; 
    }
  }

  /* 
   * stop stops the time passing of the clock.
   */
  public void stop(){
    Date d = new Date(); // Use System.currentTimeMillis();
    if (!stopped){
      stopped = true;
      long stoppedAt = d.getTime();
      currentValue = stoppedAt - startedAt;
    }
  }

  /* 
   * value returns the current value of the clock.
   */
  public long value(){
    Date d = new Date(); // Use System.currentTimeMillis();
    if (stopped) 
      return currentValue;
    else
      return (d.getTime() - startedAt);
  }

  public String toString(){
    long val = value();
    Date d = new Date(val);
    // The getHours/getMinutes/getSeconds methods are deprecated.
    // Should eventually use these:
    // Calendar.get(Calendar.HOUR_OF_DAY). 
    // Calendar.get(Calendar.MINUTE).
    // Calendar.get(Calendar.SECOND). 
    return ("("+val+")"+Integer.toString(d.getHours()-18)+":"+
	    Integer.toString(d.getMinutes())+":"+
	    Integer.toString(d.getSeconds()));
  }

}



















