/*
 * Chess game implementation.
 * By Carlos Varela.  February, 1997
 *
 * MoveTree class.  Contains a tree of valid moves in the game.
 */

package chess;

import java.util.*;

public class MoveTree implements submit.MoveTree, java.io.Serializable  {

  int value;             // The value of the static function applied to
                         // the board position after this move.
  int maxMin;            // The value of the maximum minimum subtree.
  MoveTree maxMinTree;   // The subtree with best performance.
  Vector nextMoves;      // A vector of MoveTrees representing the
                         // children of this move.

  Move move;             // The move that this node represents

  public void setMove(submit.Move move){
    if (move instanceof chess.Move) this.move = (chess.Move) move;
  }

  public Move getMove(){
    return move;
  }

  public MoveTree(Move m, int v, int mm, MoveTree mmt, Vector next){
    setMove(m);
    value = v;
    maxMin = mm;
    maxMinTree = mmt;
    nextMoves = next;
  }

  public int getValue(){
    return value;
  }

  public void setValue(int value){
    this.value = value;
  }

  public int getMaxMin(){
    return maxMin;
  }

  public void setMaxMin(int maxMin){
    this.maxMin = maxMin;
  }

  public MoveTree getMaxMinTree(){
    return maxMinTree;
  }

  public void setMaxMinTree(MoveTree maxMinTree){
    this.maxMinTree = maxMinTree;
  }

  public Vector getNextMoves(){
    return nextMoves;
  }

  public void addNextMove(MoveTree nextMove){
    if (nextMoves == null)
      nextMoves = new Vector(50);
    nextMoves.addElement(nextMove);
  }

  /** depth decides in how many moves has the current minmax estimate been made
   **/
  public int depth(){
    int d = 1;
    MoveTree tree = maxMinTree;
    while (tree != null){
      d++;
      tree = tree.getMaxMinTree();
    }
    return d;
  }

  /**
   * whitePosition decides the position of a new element in
   * a vector already sorted from max to min.
   */
  public int whitePosition(Vector v, int val, int from, int to){
    int position = 0;
    try{
      while (((MoveTree)v.elementAt(position)).getMaxMin() >= val){
	position++;
      }
    } catch (IndexOutOfBoundsException e){}
    return position;
  }

  public int blackPosition(Vector v, int val, int from, int to){
    int position = 0;
    try{
      while (((MoveTree)v.elementAt(position)).getMaxMin() <= val){
	position++;
      }
    } catch (IndexOutOfBoundsException e){}
    return position;
  }

//   public int whitePosition(Vector v, int val, int from, int to){
//     if (from >= to) return from;
//     else {
//       int half = (from+to)/2;
//       int halfVal = ((MoveTree)v.elementAt(half)).getMaxMin();
//       if (halfVal == val)
// 	return half;
//       else if (val < halfVal)
// 	return whitePosition(v,val,half+1,to);
//       else
// 	return whitePosition(v,val,from,half-1);
//     }
//   }

//   public int blackPosition(Vector v, int val, int from, int to){
//     if (from >= to) return from;
//     else {
//       int half = (from+to)/2;
//       int halfVal = ((MoveTree)v.elementAt(half)).getMaxMin();
//       if (halfVal == val)
// 	return half;
//       else if (val > halfVal)
// 	return blackPosition(v,val,half+1,to);
//       else
// 	return blackPosition(v,val,from,half-1);
//     }
//   }

  public void addNextMoveSorted(boolean color, MoveTree nextMove){
    if (nextMoves == null)
      nextMoves = new Vector(50);
    int size = nextMoves.size();
    int place = (color?
		 whitePosition(nextMoves, nextMove.getMaxMin(), 0, size):
		 blackPosition(nextMoves, nextMove.getMaxMin(), 0, size));
    if (place <= size)
      nextMoves.insertElementAt(nextMove, place);
    else
      nextMoves.addElement(nextMove);
  }

  public Vector prune() {
    return prune(0);
  }
  /**
   * prune returns a Vector of Moves representing the next possible moves
   * in order of preference (according to their minimax value in this move tree).
   * If x-window>0, some moves are ignored.
   * If color is WHITE, return ONLY:
   *    getMaxMin() >= maxMin - x-window.
   * If color is BLACK, return ONLY:
   *    getMaxMin() <= maxMin + x-window.
   */
  Vector prune(int xwindow){
    // FIX:  For now ignores, x-window.
    if (nextMoves == null)
      return new Vector(0);
    Vector moves = new Vector(nextMoves.size());
    for (Enumeration en = nextMoves.elements();
	 en.hasMoreElements();){
      moves.addElement(((MoveTree)en.nextElement()).getMove());
    }
    //cvarela: why destroy tree of best moves?
    //nextMoves = null;
    return moves;
  }

  public String toString(){
      return("("+getMove().toString()+","+Float.toString(value)
	     +","+Float.toString(maxMin)+","+nextMoves+")");
  
  }

}
