/*
 * Chess game implementation.
 * By Carlos Varela.  February, 1997
 *
 * Pawn class.  Contains a chess piece representing a pawn.
 */

package chess;

import java.util.*;

class Pawn extends Piece implements java.io.Serializable {

  Pawn(boolean col){
    value = (col? 100 : -100);
    color = col;
  }

  Vector generateMoves(){
    Vector pawnMoves = new Vector(12);
    char c = getColumn();
    int r = getRow();
    Board b = getBoard();
    Piece p;
    if (color() == Game.WHITE){  // White pawn
      if (!b.pieceAt(c,r+1)){    // No piece is in front.
	if (r == 7){             // To the last row!
	  pawnMoves.addElement(new Move(this, c, r, c, r+1, null, 
					new Queen(Game.WHITE)));
	  pawnMoves.addElement(new Move(this, c, r, c, r+1, null, 
					new Rook(Game.WHITE)));
	  pawnMoves.addElement(new Move(this, c, r, c, r+1, null, 
					new Bishop(Game.WHITE)));
	  pawnMoves.addElement(new Move(this, c, r, c, r+1, null, 
					new Knight(Game.WHITE)));
	} else {
	  pawnMoves.addElement(new Move(this, c, r, c, r+1));
	  if (r == 2)            // Pawn at starting position.
	    if (!b.pieceAt(c, r+2))
	      pawnMoves.addElement(new Move(this, c, r, c, r+2));
	}
      }
      try {                      // Can eat left diagonally?
	if (((p = (b.getPieceAt((char)(c-1),r+1))) != null) &&
	    (p.color() == Game.BLACK)){
	  if (r == 7){             // To the last row!
	    pawnMoves.addElement(new Move(this, c, r, (char)(c-1), r+1, p, 
					  new Queen(Game.WHITE)));
	    pawnMoves.addElement(new Move(this, c, r, (char)(c-1), r+1, p, 
					  new Rook(Game.WHITE)));
	    pawnMoves.addElement(new Move(this, c, r, (char)(c-1), r+1, p, 
					  new Bishop(Game.WHITE)));
	    pawnMoves.addElement(new Move(this, c, r, (char)(c-1), r+1, p, 
					  new Knight(Game.WHITE)));
	  } else 
	    pawnMoves.addElement(new Move(this, c, r, (char)(c-1), r+1));
	}
      } catch (ArrayIndexOutOfBoundsException e){};
      try {                      // Can eat right diagonally?
	if (((p = (b.getPieceAt((char)(c+1),r+1))) != null) &&
	    (p.color() == Game.BLACK)){
	  if (r == 7){           // To the last row!
	    pawnMoves.addElement(new Move(this, c, r, (char)(c+1), r+1, p, 
					  new Queen(Game.WHITE)));
	    pawnMoves.addElement(new Move(this, c, r, (char)(c+1), r+1, p, 
					  new Rook(Game.WHITE)));
	    pawnMoves.addElement(new Move(this, c, r, (char)(c+1), r+1, p, 
					  new Bishop(Game.WHITE)));
	    pawnMoves.addElement(new Move(this, c, r, (char)(c+1), r+1, p, 
					  new Knight(Game.WHITE)));
	  } else 
	    pawnMoves.addElement(new Move(this, c, r, (char)(c+1), r+1));
	}
      } catch (ArrayIndexOutOfBoundsException e){};
      if (r == 5){               // "Pawn-in-the-way"?
	try {
	  Move lastMove = b.lastMove();
	  if (((p = lastMove.getMovingPiece()) instanceof Pawn) &&
	      (lastMove.getInitialRow() == 7) &&
	      (lastMove.getFinalRow() == 5) &&
	      (((lastMove.getInitialColumn() == c-1) &&
		(lastMove.getFinalColumn() == c-1)) || 
	       ((lastMove.getInitialColumn() == c+1) &&
		(lastMove.getFinalColumn() == c+1))))
	    pawnMoves.addElement(new Move(this, c, r, 
					  lastMove.getInitialColumn(),
					  6, p));
	} catch (NoSuchElementException noLastMove){ //reading from FEN
	}
      }
    } else {                     // Black pawn
      if (!b.pieceAt(c,r-1)){    // No piece is in front.
	if (r == 2){             // To the last row!
	  pawnMoves.addElement(new Move(this, c, r, c, r-1, null, 
					new Queen(Game.BLACK)));
	  pawnMoves.addElement(new Move(this, c, r, c, r-1, null, 
					new Rook(Game.BLACK)));
	  pawnMoves.addElement(new Move(this, c, r, c, r-1, null, 
					new Bishop(Game.BLACK)));
	  pawnMoves.addElement(new Move(this, c, r, c, r-1, null, 
					new Knight(Game.BLACK)));
	} else {
	  pawnMoves.addElement(new Move(this, c, r, c, r-1));
	  if (r == 7)            // Pawn at starting position.
	    if (!b.pieceAt(c, r-2))
	      pawnMoves.addElement(new Move(this, c, r, c, r-2));
	}
      }
      try {                      // Can eat left diagonally?
	if (((p = (b.getPieceAt((char)(c-1),r-1))) != null) &&
	    (p.color() == Game.WHITE)){
	  if (r == 2){           // To the last row!
	    pawnMoves.addElement(new Move(this, c, r, (char)(c-1), r-1, p, 
					  new Queen(Game.BLACK)));
	    pawnMoves.addElement(new Move(this, c, r, (char)(c-1), r-1, p, 
					  new Rook(Game.BLACK)));
	    pawnMoves.addElement(new Move(this, c, r, (char)(c-1), r-1, p, 
					  new Bishop(Game.BLACK)));
	    pawnMoves.addElement(new Move(this, c, r, (char)(c-1), r-1, p, 
					  new Knight(Game.BLACK)));
	  } else 
	    pawnMoves.addElement(new Move(this, c, r, (char)(c-1), r-1));
	}
      } catch (ArrayIndexOutOfBoundsException e){};
      try {                      // Can eat right diagonally?
	if (((p = (b.getPieceAt((char)(c+1),r-1))) != null) &&
	    (p.color() == Game.WHITE)){
	  if (r == 2){           // To the last row!
	    pawnMoves.addElement(new Move(this, c, r, (char)(c+1), r-1, p, 
					  new Queen(Game.BLACK)));
	    pawnMoves.addElement(new Move(this, c, r, (char)(c+1), r-1, p, 
					  new Rook(Game.BLACK)));
	    pawnMoves.addElement(new Move(this, c, r, (char)(c+1), r-1, p, 
					  new Bishop(Game.BLACK)));
	    pawnMoves.addElement(new Move(this, c, r, (char)(c+1), r-1, p, 
					  new Knight(Game.BLACK)));
	  } else 
	    pawnMoves.addElement(new Move(this, c, r, (char)(c+1), r-1));
	}
      } catch (ArrayIndexOutOfBoundsException e){};
      if (r == 4){               // "Pawn-in-the-way"?
	try {
	  Move lastMove = b.lastMove();
	  if (((p = lastMove.getMovingPiece()) instanceof Pawn) &&
	      (lastMove.getInitialRow() == 2) &&
	      (lastMove.getFinalRow() == 4) &&
	      (((lastMove.getInitialColumn() == c-1) &&
		(lastMove.getFinalColumn() == c-1)) || 
	       ((lastMove.getInitialColumn() == c+1) &&
		(lastMove.getFinalColumn() == c+1))))
	    pawnMoves.addElement(new Move(this, c, r, 
					  lastMove.getInitialColumn(),
					  3, p));
 	} catch (NoSuchElementException noLastMove){ //reading from FEN
	}
     }
    }
    return pawnMoves;
  }

  boolean isProtected(){
    // Incomplete.
    return true;
  }

  boolean attacksCenter(){
    // Incomplete.
    return false;
  }

  boolean makesCheck(){
    // Incomplete.
    return false;
  }

  public String toString(){
    return ((color == Game.WHITE)? "P" : "p");
  }

}
