/*
 * Chess game implementation.
 * By Carlos Varela.  February, 1997
 *
 * Queen class.  Contains a chess piece representing a queen.
 */

package chess;

import java.util.*;

class Queen extends Piece implements java.io.Serializable {

  Queen(boolean col){
    value = (col? 900 : -900);
    color = col;
  }

  Vector generateMoves(){
    Vector queenMoves = new Vector(27);
    char c = getColumn();
    int r = getRow();
    Board b = getBoard();
    Piece p;
    // Queen movements
    
                                 // Right Upward moves 
    try {
      for (int i = 1; i <= 7; i++){
	int ri = r + i;
	char ci = (char) (c + i);
	if ((p = b.getPieceAt(ci,ri)) != null){  // A piece is in the way
	  if (p.color() != color())              // Can eat if opposite color.
	    queenMoves.addElement(new Move(this, c, r, ci, ri));
	  break;
	} else
	  queenMoves.addElement(new Move(this, c, r, ci, ri));
      }
    } catch (ArrayIndexOutOfBoundsException e){};  /* Out of board */
	 
                                 // Right Downward moves 
    try {
      for (int i = 1; i <= 7; i++){
	int ri = r - i;
	char ci = (char) (c + i);
	if ((p = b.getPieceAt(ci,ri)) != null){  // A piece is in the way
	  if (p.color() != color())            // Can eat if opposite color.
	    queenMoves.addElement(new Move(this, c, r, ci, ri));
	  break;
	} else
	  queenMoves.addElement(new Move(this, c, r, ci, ri));
      }
    } catch (ArrayIndexOutOfBoundsException e){};  /* Out of board */

                                 // Left Upward moves 
    try {
      for (int i = 1; i <= 7; i++){
	int ri = r + i;
	char ci = (char) (c - i);
	if ((p = b.getPieceAt(ci,ri)) != null){  // A piece is in the way
	  if (p.color() != color())            // Can eat if opposite color.
	    queenMoves.addElement(new Move(this, c, r, ci, ri));
	  break;
	} else
	  queenMoves.addElement(new Move(this, c, r, ci, ri));
      }
    } catch (ArrayIndexOutOfBoundsException e){};  /* Out of board */
	 
                                 // Left Downward moves 
    try {
      for (int i = 1; i <= 7; i++){
	int ri = r - i;
	char ci = (char) (c - i);
	if ((p = b.getPieceAt(ci,ri)) != null){  // A piece is in the way
	  if (p.color() != color())            // Can eat if opposite color.
	    queenMoves.addElement(new Move(this, c, r, ci, ri));
	  break;
	} else
	  queenMoves.addElement(new Move(this, c, r, ci, ri));
      }
    } catch (ArrayIndexOutOfBoundsException e){};  /* Out of board */
    
                                 // Upward moves 
    for (int ri = r+1; ri <= 8; ri++){
      if ((p = b.getPieceAt(c,ri)) != null){  // A piece is in the way
	if (p.color() != color())           // Can eat if opposite color.
	  queenMoves.addElement(new Move(this, c, r, c, ri));
	break;
      } else
	queenMoves.addElement(new Move(this, c, r, c, ri));
    }
	 
                                 // Downward moves 
    for (int ri = r-1; ri >= 1; ri--){
      if ((p = b.getPieceAt(c,ri)) != null){  // A piece is in the way
	if (p.color() != color())           // Can eat if opposite color.
	  queenMoves.addElement(new Move(this, c, r, c, ri));
	break;
      } else
	queenMoves.addElement(new Move(this, c, r, c, ri));
    }
	 
                                 // Leftward moves 
    for (char ci = (char) (c-1); ci >= 'a'; ci--){
      if ((p = b.getPieceAt(ci,r)) != null){  // A piece is in the way
	if (p.color() != color())           // Can eat if opposite color.
	  queenMoves.addElement(new Move(this, c, r, ci, r));
	break;
      } else
	queenMoves.addElement(new Move(this, c, r, ci, r));
    }

                                 // Rightward moves 
    for (char ci = (char) (c+1); ci <= 'h'; ci++){
      if ((p = b.getPieceAt(ci,r)) != null){  // A piece is in the way
	if (p.color() != color())           // Can eat if opposite color.
	  queenMoves.addElement(new Move(this, c, r, ci, r));
	break;
      } else
	queenMoves.addElement(new Move(this, c, r, ci, r));
    }

    return queenMoves;
  }

  boolean isProtected(){
    // Incomplete.
    return true;
  }

  boolean attacksCenter(){
    // Incomplete.
    return false;
  }

  boolean makesCheck(){
    // Incomplete.
    return false;
  }

  public String toString(){
    return ((color == Game.WHITE)? "Q" : "q");
  }

}
